import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {ProductGamesystem} from "../../../../game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../../../game-model/gamesystems/SimpleGamesystem";
import {
  MatCell, MatCellDef,
  MatColumnDef, MatHeaderCell, MatHeaderCellDef,
  MatHeaderRow,
  MatHeaderRowDef,
  MatRow,
  MatRowDef,
  MatTable,
  MatTableDataSource
} from "@angular/material/table";
import {SimpleState} from "../../../../game-model/gamesystems/SimpleState";
import {State} from "../../../../game-model/gamesystems/State";
import {NgForOf, NgIf} from "@angular/common";
import {ProductState} from "../../../../game-model/gamesystems/ProductState";
import {MatIcon} from "@angular/material/icon";
import {Gamesystem} from "../../../../game-model/gamesystems/Gamesystem";
import {LeafGamesystemCalculator} from "../../product-gamesystem-editor/LeafGamesystemCalculator";
import {MatTooltip} from "@angular/material/tooltip";

@Component({
  selector: 'app-product-state-editor',
  standalone: true,
  imports: [
    MatTable,
    MatRow,
    MatRowDef,
    MatHeaderRow,
    MatHeaderRowDef,
    MatColumnDef,
    MatHeaderCell,
    MatHeaderCellDef,
    MatCell,
    MatCellDef,
    NgForOf,
    NgIf,
    MatIcon,
    MatTooltip
  ],
  templateUrl: './product-state-editor.component.html',
  styleUrl: './product-state-editor.component.scss'
})
export class ProductStateEditorComponent implements OnInit{

  @Input() gamesystem: ProductGamesystem | undefined
  @Output('onOpenGamesystemEditor') openGamesystemEditorEmitter = new EventEmitter<SimpleGamesystem>();
  displayedColumns: string[] = [];
  datasource = new MatTableDataSource();


  ngOnInit() {
    this.gamesystem!.generateFromChildsystems();
    this.generateColumnNamesRecursively(this.gamesystem!, "");
    this.displayedColumns.push('Initial');
    this.datasource.data = this.gamesystem!.states;
  }

  generateColumnNamesRecursively(gamesystem: ProductGamesystem, nestedColumnName: string) {
    gamesystem.innerGamesystems.forEach(innerGamesystem => {
      if(innerGamesystem instanceof SimpleGamesystem) {
        this.displayedColumns.push(nestedColumnName + innerGamesystem.componentName);
      } else {
        this.generateColumnNamesRecursively(innerGamesystem as ProductGamesystem, nestedColumnName + innerGamesystem.componentName + ".");
      }
    })
  }

  protected readonly SimpleState = SimpleState;

  getLeafState(state: State<any>, i: number) {
    return LeafGamesystemCalculator.calcLeafStates(state)[i];
  }

  clickOnInnerState(leafIndex: number) {
    const leaf_gamesystems = LeafGamesystemCalculator.calcLeafGeamesystems(this.gamesystem!);
    const clicked_gamesystem = leaf_gamesystems[leafIndex];
    this.openGamesystemEditorEmitter.emit(clicked_gamesystem);
  }

}
