import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {ProductGamesystem} from "../../../../game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../../../game-model/gamesystems/SimpleGamesystem";
import {
  MatCell, MatCellDef,
  MatColumnDef,
  MatHeaderCell,
  MatHeaderCellDef, MatHeaderRow, MatHeaderRowDef, MatRow, MatRowDef,
  MatTable,
  MatTableDataSource
} from "@angular/material/table";
import {LeafGamesystemCalculator} from "../../product-gamesystem-editor/LeafGamesystemCalculator";
import {NgForOf, NgIf} from "@angular/common";
import {ProductTransition} from "../../../../game-model/gamesystems/ProductTransition";
import {ProductState} from "../../../../game-model/gamesystems/ProductState";
import {MatTooltip} from "@angular/material/tooltip";

class DisplayedColumnName {
  displayedName: string
  internalName: string


  constructor(displayedName: string, internalName: string) {
    this.displayedName = displayedName;
    this.internalName = internalName;
  }
}
@Component({
  selector: 'app-product-transition-editor',
  standalone: true,
  imports: [
    MatTable,
    NgForOf,
    MatColumnDef,
    MatHeaderCell,
    MatHeaderCellDef,
    MatCell,
    MatCellDef,
    NgIf,
    MatHeaderRow,
    MatRow,
    MatHeaderRowDef,
    MatRowDef,
    MatTooltip
  ],
  templateUrl: './product-transition-editor.component.html',
  styleUrl: './product-transition-editor.component.scss'
})
export class ProductTransitionEditorComponent implements OnInit{

  @Input() gamesystem: ProductGamesystem | undefined
  @Output() onOpenGamesystem = new EventEmitter<SimpleGamesystem>();

  dataSource = new MatTableDataSource();
  displayedColumns: DisplayedColumnName[] = [];
  columns: string[] = [];
  numberLeafSystems: number = -1;

  ngOnInit() {
    if(this.gamesystem != undefined) {
      const leafGamesystems: SimpleGamesystem[] = LeafGamesystemCalculator.calcLeafGeamesystems(this.gamesystem);
      this.displayedColumns = leafGamesystems.map(leafGamesystem => new DisplayedColumnName(leafGamesystem.componentName, leafGamesystem.componentName + "-start"));
      this.displayedColumns = this.displayedColumns.concat( leafGamesystems.map(leafGamesystem => new DisplayedColumnName(leafGamesystem.componentName, leafGamesystem.componentName + "-end")));

      this.numberLeafSystems = leafGamesystems.length;
      this.columns = this.displayedColumns.map(column => column.internalName)

      this.dataSource.data = this.gamesystem.transitions;
    }
  }

  getLeafStateByIndex(transition: ProductTransition, leafIndex: number) {
    let state: ProductState;
    let index = leafIndex;
    if(leafIndex < this.numberLeafSystems) {
      state = transition.startingState
    } else {
      state = transition.endingState;
      index = leafIndex - this.numberLeafSystems;
    }



    const leafStates = LeafGamesystemCalculator.calcLeafStates(state);
    console.log(leafStates)
    return leafStates[index];
  }

  openGamesystemEditor(leafIndex: number) {
    const leafGamesystems = LeafGamesystemCalculator.calcLeafGeamesystems(this.gamesystem!);
    if(leafIndex < this.numberLeafSystems) {
      this.onOpenGamesystem.emit(leafGamesystems[leafIndex])
    } else {
      this.onOpenGamesystem.emit(leafGamesystems[leafIndex - this.numberLeafSystems])
    }
  }
}
